#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import datetime
from unittest import mock

from novaclient import api_versions

from openstackclient.compute.v2 import usage as usage_cmds
from openstackclient.tests.unit.compute.v2 import fakes as compute_fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes


class TestUsage(compute_fakes.TestComputev2):

    def setUp(self):
        super(TestUsage, self).setUp()

        self.usage_mock = self.app.client_manager.compute.usage
        self.usage_mock.reset_mock()

        self.projects_mock = self.app.client_manager.identity.projects
        self.projects_mock.reset_mock()


class TestUsageList(TestUsage):

    project = identity_fakes.FakeProject.create_one_project()
    # Return value of self.usage_mock.list().
    usages = compute_fakes.FakeUsage.create_usages(
        attrs={'tenant_id': project.name}, count=1)

    columns = (
        "Project",
        "Servers",
        "RAM MB-Hours",
        "CPU Hours",
        "Disk GB-Hours"
    )

    data = [(
        usage_cmds.ProjectColumn(usages[0].tenant_id),
        usage_cmds.CountColumn(usages[0].server_usages),
        usage_cmds.FloatColumn(usages[0].total_memory_mb_usage),
        usage_cmds.FloatColumn(usages[0].total_vcpus_usage),
        usage_cmds.FloatColumn(usages[0].total_local_gb_usage),
    )]

    def setUp(self):
        super(TestUsageList, self).setUp()

        self.usage_mock.list.return_value = self.usages

        self.projects_mock.list.return_value = [self.project]
        # Get the command object to test
        self.cmd = usage_cmds.ListUsage(self.app, None)

    def test_usage_list_no_options(self):

        arglist = []
        verifylist = [
            ('start', None),
            ('end', None),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.projects_mock.list.assert_called_with()

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(tuple(self.data), tuple(data))

    def test_usage_list_with_options(self):
        arglist = [
            '--start', '2016-11-11',
            '--end', '2016-12-20',
        ]
        verifylist = [
            ('start', '2016-11-11'),
            ('end', '2016-12-20'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.projects_mock.list.assert_called_with()
        self.usage_mock.list.assert_called_with(
            datetime.datetime(2016, 11, 11, 0, 0),
            datetime.datetime(2016, 12, 20, 0, 0),
            detailed=True)

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(tuple(self.data), tuple(data))

    def test_usage_list_with_pagination(self):
        arglist = []
        verifylist = [
            ('start', None),
            ('end', None),
        ]

        self.app.client_manager.compute.api_version = api_versions.APIVersion(
            '2.40')
        self.usage_mock.list.reset_mock()
        self.usage_mock.list.side_effect = [self.usages, []]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.projects_mock.list.assert_called_with()
        self.usage_mock.list.assert_has_calls([
            mock.call(mock.ANY, mock.ANY, detailed=True),
            mock.call(mock.ANY, mock.ANY, detailed=True,
                      marker=self.usages[0]['server_usages'][0]['instance_id'])
        ])
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(tuple(self.data), tuple(data))


class TestUsageShow(TestUsage):

    project = identity_fakes.FakeProject.create_one_project()
    # Return value of self.usage_mock.list().
    usage = compute_fakes.FakeUsage.create_one_usage(
        attrs={'tenant_id': project.name})

    columns = (
        'Project',
        'Servers',
        'RAM MB-Hours',
        'CPU Hours',
        'Disk GB-Hours',
    )

    data = (
        usage_cmds.ProjectColumn(usage.tenant_id),
        usage_cmds.CountColumn(usage.server_usages),
        usage_cmds.FloatColumn(usage.total_memory_mb_usage),
        usage_cmds.FloatColumn(usage.total_vcpus_usage),
        usage_cmds.FloatColumn(usage.total_local_gb_usage),
    )

    def setUp(self):
        super(TestUsageShow, self).setUp()

        self.usage_mock.get.return_value = self.usage

        self.projects_mock.get.return_value = self.project
        # Get the command object to test
        self.cmd = usage_cmds.ShowUsage(self.app, None)

    def test_usage_show_no_options(self):

        self.app.client_manager.auth_ref = mock.Mock()
        self.app.client_manager.auth_ref.project_id = self.project.id

        arglist = []
        verifylist = [
            ('project', None),
            ('start', None),
            ('end', None),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_usage_show_with_options(self):

        arglist = [
            '--project', self.project.id,
            '--start', '2016-11-11',
            '--end', '2016-12-20',
        ]
        verifylist = [
            ('project', self.project.id),
            ('start', '2016-11-11'),
            ('end', '2016-12-20'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.usage_mock.get.assert_called_with(
            self.project.id,
            datetime.datetime(2016, 11, 11, 0, 0),
            datetime.datetime(2016, 12, 20, 0, 0))

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from unittest import mock

from openstackclient.identity.v3 import role_assignment
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes


class TestRoleAssignment(identity_fakes.TestIdentityv3):

    def setUp(self):
        super(TestRoleAssignment, self).setUp()


class TestRoleAssignmentList(TestRoleAssignment):

    columns = (
        'Role',
        'User',
        'Group',
        'Project',
        'Domain',
        'System',
        'Inherited',
    )

    def setUp(self):
        super(TestRoleAssignment, self).setUp()

        # Get a shortcut to the UserManager Mock
        self.users_mock = self.app.client_manager.identity.users
        self.users_mock.reset_mock()

        # Get a shortcut to the GroupManager Mock
        self.groups_mock = self.app.client_manager.identity.groups
        self.groups_mock.reset_mock()

        # Get a shortcut to the DomainManager Mock
        self.domains_mock = self.app.client_manager.identity.domains
        self.domains_mock.reset_mock()

        # Get a shortcut to the ProjectManager Mock
        self.projects_mock = self.app.client_manager.identity.projects
        self.projects_mock.reset_mock()

        # Get a shortcut to the RoleManager Mock
        self.roles_mock = self.app.client_manager.identity.roles
        self.roles_mock.reset_mock()

        self.role_assignments_mock = self.app.client_manager.identity.\
            role_assignments
        self.role_assignments_mock.reset_mock()

        # Get the command object to test
        self.cmd = role_assignment.ListRoleAssignment(self.app, None)

    def test_role_assignment_list_no_filters(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_GROUP_ID),
                loaded=True,
            ),
        ]

        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            group=None,
            effective=False,
            role=None,
            user=None,
            project=None,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            identity_fakes.project_id,
            '',
            '',
            False
        ), (identity_fakes.role_id,
            '',
            identity_fakes.group_id,
            identity_fakes.project_id,
            '',
            '',
            False
            ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_user(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_DOMAIN_ID_AND_USER_ID),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID),
                loaded=True,
            ),
        ]

        arglist = [
            '--user', identity_fakes.user_name
        ]
        verifylist = [
            ('user', identity_fakes.user_name),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', None),
            ('effective', False),
            ('inherited', False),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            user=self.users_mock.get(),
            group=None,
            project=None,
            role=None,
            effective=False,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            '',
            identity_fakes.domain_id,
            '',
            False
        ), (identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            identity_fakes.project_id,
            '',
            '',
            False
            ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_group(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_DOMAIN_ID_AND_GROUP_ID),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_GROUP_ID),
                loaded=True,
            ),
        ]

        arglist = [
            '--group', identity_fakes.group_name
        ]
        verifylist = [
            ('user', None),
            ('group', identity_fakes.group_name),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', None),
            ('effective', False),
            ('inherited', False),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            group=self.groups_mock.get(),
            effective=False,
            project=None,
            role=None,
            user=None,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            '',
            identity_fakes.group_id,
            '',
            identity_fakes.domain_id,
            '',
            False
        ), (identity_fakes.role_id,
            '',
            identity_fakes.group_id,
            identity_fakes.project_id,
            '',
            '',
            False
            ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_domain(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_DOMAIN_ID_AND_USER_ID),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_DOMAIN_ID_AND_GROUP_ID),
                loaded=True,
            ),
        ]

        arglist = [
            '--domain', identity_fakes.domain_name
        ]
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', identity_fakes.domain_name),
            ('project', None),
            ('role', None),
            ('effective', False),
            ('inherited', False),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=self.domains_mock.get(),
            system=None,
            group=None,
            effective=False,
            project=None,
            role=None,
            user=None,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            '',
            identity_fakes.domain_id,
            '',
            False
        ), (identity_fakes.role_id,
            '',
            identity_fakes.group_id,
            '',
            identity_fakes.domain_id,
            '',
            False
            ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_project(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_GROUP_ID),
                loaded=True,
            ),
        ]

        arglist = [
            '--project', identity_fakes.project_name
        ]
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', identity_fakes.project_name),
            ('role', None),
            ('effective', False),
            ('inherited', False),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            group=None,
            effective=False,
            project=self.projects_mock.get(),
            role=None,
            user=None,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            identity_fakes.project_id,
            '',
            '',
            False
        ), (identity_fakes.role_id,
            '',
            identity_fakes.group_id,
            identity_fakes.project_id,
            '',
            '',
            False
            ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_def_creds(self):

        auth_ref = self.app.client_manager.auth_ref = mock.Mock()
        auth_ref.project_id.return_value = identity_fakes.project_id
        auth_ref.user_id.return_value = identity_fakes.user_id

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID),
                loaded=True,
            ),
        ]

        arglist = [
            '--auth-user',
            '--auth-project',
        ]
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', None),
            ('effective', False),
            ('inherited', False),
            ('names', False),
            ('authuser', True),
            ('authproject', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            user=self.users_mock.get(),
            group=None,
            project=self.projects_mock.get(),
            role=None,
            effective=False,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            identity_fakes.project_id,
            '',
            '',
            False
        ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_effective(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_DOMAIN_ID_AND_USER_ID),
                loaded=True,
            ),
        ]

        arglist = ['--effective']
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', None),
            ('effective', True),
            ('inherited', False),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            group=None,
            effective=True,
            project=None,
            role=None,
            user=None,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            identity_fakes.project_id,
            '',
            '',
            False
        ), (identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            '',
            identity_fakes.domain_id,
            '',
            False
            ),)
        self.assertEqual(tuple(data), datalist)

    def test_role_assignment_list_inherited(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    (identity_fakes.
                        ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID_INHERITED)),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    (identity_fakes.
                        ASSIGNMENT_WITH_DOMAIN_ID_AND_USER_ID_INHERITED)),
                loaded=True,
            ),
        ]

        arglist = ['--inherited']
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', None),
            ('effective', False),
            ('inherited', True),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            group=None,
            effective=False,
            project=None,
            role=None,
            user=None,
            os_inherit_extension_inherited_to='projects',
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            identity_fakes.project_id,
            '',
            '',
            True
        ), (identity_fakes.role_id,
            identity_fakes.user_id,
            '',
            '',
            identity_fakes.domain_id,
            '',
            True
            ),)
        self.assertEqual(datalist, tuple(data))

    def test_role_assignment_list_include_names(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes
                    .ASSIGNMENT_WITH_PROJECT_ID_AND_USER_ID_INCLUDE_NAMES),
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes
                    .ASSIGNMENT_WITH_DOMAIN_ID_AND_USER_ID_INCLUDE_NAMES),
                loaded=True,
            ),
        ]

        arglist = ['--names']
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', None),
            ('effective', False),
            ('inherited', False),
            ('names', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # DisplayCommandBase.take_action() returns two tuples

        # This test will not run correctly until the patch in the python
        # client is merged. Once that is done 'data' should return the
        # correct information
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            group=None,
            effective=False,
            project=None,
            role=None,
            user=None,
            os_inherit_extension_inherited_to=None,
            include_names=True)

        collist = (
            'Role', 'User', 'Group', 'Project', 'Domain', 'System', 'Inherited'
        )
        self.assertEqual(columns, collist)

        datalist1 = ((
            identity_fakes.role_name,
            '@'.join([identity_fakes.user_name, identity_fakes.domain_name]),
            '',
            '@'.join([identity_fakes.project_name,
                      identity_fakes.domain_name]),
            '',
            '',
            False
        ), (identity_fakes.role_name,
            '@'.join([identity_fakes.user_name, identity_fakes.domain_name]),
            '',
            '',
            identity_fakes.domain_name,
            '',
            False
            ),)
        self.assertEqual(tuple(data), datalist1)

    def test_role_assignment_list_domain_role(self):

        self.role_assignments_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(
                    identity_fakes.ASSIGNMENT_WITH_DOMAIN_ROLE),
                loaded=True,
            ),
        ]

        arglist = [
            '--role', identity_fakes.ROLE_2['name'],
            '--role-domain', identity_fakes.domain_name
        ]
        verifylist = [
            ('user', None),
            ('group', None),
            ('system', None),
            ('domain', None),
            ('project', None),
            ('role', identity_fakes.ROLE_2['name']),
            ('effective', False),
            ('inherited', False),
            ('names', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.role_assignments_mock.list.assert_called_with(
            domain=None,
            system=None,
            user=None,
            group=None,
            project=None,
            role=self.roles_mock.get(),
            effective=False,
            os_inherit_extension_inherited_to=None,
            include_names=False)

        self.assertEqual(self.columns, columns)
        datalist = ((
            identity_fakes.ROLE_2['id'],
            identity_fakes.user_id,
            '',
            '',
            identity_fakes.domain_id,
            '',
            False
        ),)
        self.assertEqual(datalist, tuple(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from cinderclient import api_versions
from osc_lib import exceptions

from openstackclient.tests.unit.volume.v3 import fakes as volume_fakes
from openstackclient.volume.v3 import volume_group


class TestVolumeGroup(volume_fakes.TestVolume):

    def setUp(self):
        super().setUp()

        self.volume_groups_mock = self.app.client_manager.volume.groups
        self.volume_groups_mock.reset_mock()

        self.volume_group_types_mock = \
            self.app.client_manager.volume.group_types
        self.volume_group_types_mock.reset_mock()

        self.volume_types_mock = self.app.client_manager.volume.volume_types
        self.volume_types_mock.reset_mock()


class TestVolumeGroupCreate(TestVolumeGroup):

    fake_volume_type = volume_fakes.FakeVolumeType.create_one_volume_type()
    fake_volume_group_type = \
        volume_fakes.FakeVolumeGroupType.create_one_volume_group_type()
    fake_volume_group = volume_fakes.FakeVolumeGroup.create_one_volume_group(
        attrs={
            'group_type': fake_volume_group_type.id,
            'volume_types': [fake_volume_type.id],
        },
    )

    columns = (
        'ID',
        'Status',
        'Name',
        'Description',
        'Group Type',
        'Volume Types',
        'Availability Zone',
        'Created At',
        'Volumes',
        'Group Snapshot ID',
        'Source Group ID',
    )
    data = (
        fake_volume_group.id,
        fake_volume_group.status,
        fake_volume_group.name,
        fake_volume_group.description,
        fake_volume_group.group_type,
        fake_volume_group.volume_types,
        fake_volume_group.availability_zone,
        fake_volume_group.created_at,
        fake_volume_group.volumes,
        fake_volume_group.group_snapshot_id,
        fake_volume_group.source_group_id,
    )

    def setUp(self):
        super().setUp()

        self.volume_types_mock.get.return_value = self.fake_volume_type
        self.volume_group_types_mock.get.return_value = \
            self.fake_volume_group_type
        self.volume_groups_mock.create.return_value = self.fake_volume_group
        self.volume_groups_mock.get.return_value = self.fake_volume_group

        self.cmd = volume_group.CreateVolumeGroup(self.app, None)

    def test_volume_group_create(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.13')

        arglist = [
            self.fake_volume_group_type.id,
            self.fake_volume_type.id,
        ]
        verifylist = [
            ('volume_group_type', self.fake_volume_group_type.id),
            ('volume_types', [self.fake_volume_type.id]),
            ('name', None),
            ('description', None),
            ('availability_zone', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.volume_group_types_mock.get.assert_called_once_with(
            self.fake_volume_group_type.id)
        self.volume_types_mock.get.assert_called_once_with(
            self.fake_volume_type.id)
        self.volume_groups_mock.create.assert_called_once_with(
            self.fake_volume_group_type.id,
            self.fake_volume_type.id,
            None,
            None,
            availability_zone=None,
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_volume_group_create_with_options(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.13')

        arglist = [
            self.fake_volume_group_type.id,
            self.fake_volume_type.id,
            '--name', 'foo',
            '--description', 'hello, world',
            '--availability-zone', 'bar',
        ]
        verifylist = [
            ('volume_group_type', self.fake_volume_group_type.id),
            ('volume_types', [self.fake_volume_type.id]),
            ('name', 'foo'),
            ('description', 'hello, world'),
            ('availability_zone', 'bar'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.volume_group_types_mock.get.assert_called_once_with(
            self.fake_volume_group_type.id)
        self.volume_types_mock.get.assert_called_once_with(
            self.fake_volume_type.id)
        self.volume_groups_mock.create.assert_called_once_with(
            self.fake_volume_group_type.id,
            self.fake_volume_type.id,
            'foo',
            'hello, world',
            availability_zone='bar',
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_volume_group_create_pre_v313(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.12')

        arglist = [
            self.fake_volume_group_type.id,
            self.fake_volume_type.id,
        ]
        verifylist = [
            ('volume_group_type', self.fake_volume_group_type.id),
            ('volume_types', [self.fake_volume_type.id]),
            ('name', None),
            ('description', None),
            ('availability_zone', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn(
            '--os-volume-api-version 3.13 or greater is required',
            str(exc))


class TestVolumeGroupDelete(TestVolumeGroup):

    fake_volume_group = \
        volume_fakes.FakeVolumeGroup.create_one_volume_group()

    def setUp(self):
        super().setUp()

        self.volume_groups_mock.get.return_value = self.fake_volume_group
        self.volume_groups_mock.delete.return_value = None

        self.cmd = volume_group.DeleteVolumeGroup(self.app, None)

    def test_volume_group_delete(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.13')

        arglist = [
            self.fake_volume_group.id,
            '--force',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('force', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.volume_groups_mock.delete.assert_called_once_with(
            self.fake_volume_group.id, delete_volumes=True,
        )
        self.assertIsNone(result)

    def test_volume_group_delete_pre_v313(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.12')

        arglist = [
            self.fake_volume_group.id,
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('force', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn(
            '--os-volume-api-version 3.13 or greater is required',
            str(exc))


class TestVolumeGroupSet(TestVolumeGroup):

    fake_volume_group = \
        volume_fakes.FakeVolumeGroup.create_one_volume_group()

    columns = (
        'ID',
        'Status',
        'Name',
        'Description',
        'Group Type',
        'Volume Types',
        'Availability Zone',
        'Created At',
        'Volumes',
        'Group Snapshot ID',
        'Source Group ID',
    )
    data = (
        fake_volume_group.id,
        fake_volume_group.status,
        fake_volume_group.name,
        fake_volume_group.description,
        fake_volume_group.group_type,
        fake_volume_group.volume_types,
        fake_volume_group.availability_zone,
        fake_volume_group.created_at,
        fake_volume_group.volumes,
        fake_volume_group.group_snapshot_id,
        fake_volume_group.source_group_id,
    )

    def setUp(self):
        super().setUp()

        self.volume_groups_mock.get.return_value = self.fake_volume_group
        self.volume_groups_mock.update.return_value = self.fake_volume_group

        self.cmd = volume_group.SetVolumeGroup(self.app, None)

    def test_volume_group_set(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.13')

        arglist = [
            self.fake_volume_group.id,
            '--name', 'foo',
            '--description', 'hello, world',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('name', 'foo'),
            ('description', 'hello, world'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.volume_groups_mock.update.assert_called_once_with(
            self.fake_volume_group.id, name='foo', description='hello, world',
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_volume_group_with_enable_replication_option(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.38')

        arglist = [
            self.fake_volume_group.id,
            '--enable-replication',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('enable_replication', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.volume_groups_mock.enable_replication.assert_called_once_with(
            self.fake_volume_group.id)
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_volume_group_set_pre_v313(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.12')

        arglist = [
            self.fake_volume_group.id,
            '--name', 'foo',
            '--description', 'hello, world',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('name', 'foo'),
            ('description', 'hello, world'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn(
            '--os-volume-api-version 3.13 or greater is required',
            str(exc))

    def test_volume_group_with_enable_replication_option_pre_v338(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.37')

        arglist = [
            self.fake_volume_group.id,
            '--enable-replication',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('enable_replication', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn(
            '--os-volume-api-version 3.38 or greater is required',
            str(exc))


class TestVolumeGroupList(TestVolumeGroup):

    fake_volume_groups = \
        volume_fakes.FakeVolumeGroup.create_volume_groups()

    columns = (
        'ID',
        'Status',
        'Name',
    )
    data = [
        (
            fake_volume_group.id,
            fake_volume_group.status,
            fake_volume_group.name,
        ) for fake_volume_group in fake_volume_groups
    ]

    def setUp(self):
        super().setUp()

        self.volume_groups_mock.list.return_value = self.fake_volume_groups

        self.cmd = volume_group.ListVolumeGroup(self.app, None)

    def test_volume_group_list(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.13')

        arglist = [
            '--all-projects',
        ]
        verifylist = [
            ('all_projects', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.volume_groups_mock.list.assert_called_once_with(
            search_opts={
                'all_tenants': True,
            },
        )
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(tuple(self.data), data)

    def test_volume_group_list_pre_v313(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.12')

        arglist = [
            '--all-projects',
        ]
        verifylist = [
            ('all_projects', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn(
            '--os-volume-api-version 3.13 or greater is required',
            str(exc))


class TestVolumeGroupFailover(TestVolumeGroup):

    fake_volume_group = \
        volume_fakes.FakeVolumeGroup.create_one_volume_group()

    def setUp(self):
        super().setUp()

        self.volume_groups_mock.get.return_value = self.fake_volume_group
        self.volume_groups_mock.failover_replication.return_value = None

        self.cmd = volume_group.FailoverVolumeGroup(self.app, None)

    def test_volume_group_failover(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.38')

        arglist = [
            self.fake_volume_group.id,
            '--allow-attached-volume',
            '--secondary-backend-id', 'foo',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('allow_attached_volume', True),
            ('secondary_backend_id', 'foo'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.volume_groups_mock.failover_replication.assert_called_once_with(
            self.fake_volume_group.id,
            allow_attached_volume=True,
            secondary_backend_id='foo',
        )
        self.assertIsNone(result)

    def test_volume_group_failover_pre_v338(self):
        self.app.client_manager.volume.api_version = \
            api_versions.APIVersion('3.37')

        arglist = [
            self.fake_volume_group.id,
            '--allow-attached-volume',
            '--secondary-backend-id', 'foo',
        ]
        verifylist = [
            ('group', self.fake_volume_group.id),
            ('allow_attached_volume', True),
            ('secondary_backend_id', 'foo'),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        exc = self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)
        self.assertIn(
            '--os-volume-api-version 3.38 or greater is required',
            str(exc))



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Identity v3 Credential action implementations"""

import logging

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _
from openstackclient.identity import common


LOG = logging.getLogger(__name__)


class CreateCredential(command.ShowOne):
    _description = _("Create new credential")

    def get_parser(self, prog_name):
        parser = super(CreateCredential, self).get_parser(prog_name)
        parser.add_argument(
            'user',
            metavar='<user>',
            help=_('user that owns the credential (name or ID)'),
        )
        parser.add_argument(
            '--type',
            default="cert",
            metavar='<type>',
            help=_('New credential type: cert, ec2, totp and so on'),
        )
        parser.add_argument(
            'data',
            metavar='<data>',
            help=_('New credential data'),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=_('Project which limits the scope of '
                   'the credential (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        user_id = utils.find_resource(identity_client.users,
                                      parsed_args.user).id
        if parsed_args.project:
            project = utils.find_resource(identity_client.projects,
                                          parsed_args.project).id
        else:
            project = None
        credential = identity_client.credentials.create(
            user=user_id,
            type=parsed_args.type,
            blob=parsed_args.data,
            project=project)

        credential._info.pop('links')
        return zip(*sorted(credential._info.items()))


class DeleteCredential(command.Command):
    _description = _("Delete credential(s)")

    def get_parser(self, prog_name):
        parser = super(DeleteCredential, self).get_parser(prog_name)
        parser.add_argument(
            'credential',
            metavar='<credential-id>',
            nargs='+',
            help=_('ID of credential(s) to delete'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        result = 0
        for i in parsed_args.credential:
            try:
                identity_client.credentials.delete(i)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete credentials with "
                          "ID '%(credential)s': %(e)s"),
                          {'credential': i, 'e': e})

        if result > 0:
            total = len(parsed_args.credential)
            msg = (_("%(result)s of %(total)s credential failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)


class ListCredential(command.Lister):
    _description = _("List credentials")

    def get_parser(self, prog_name):
        parser = super(ListCredential, self).get_parser(prog_name)
        parser.add_argument(
            '--user',
            metavar='<user>',
            help=_('Filter credentials by <user> (name or ID)'),
        )
        common.add_user_domain_option_to_parser(parser)
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_('Filter credentials by type: cert, ec2, totp and so on'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        kwargs = {}
        if parsed_args.user:
            user_id = common.find_user(
                identity_client,
                parsed_args.user,
                parsed_args.user_domain,
            ).id
            kwargs["user_id"] = user_id

        if parsed_args.type:
            kwargs["type"] = parsed_args.type

        columns = ('ID', 'Type', 'User ID', 'Blob', 'Project ID')
        column_headers = ('ID', 'Type', 'User ID', 'Data', 'Project ID')
        data = self.app.client_manager.identity.credentials.list(**kwargs)
        return (column_headers,
                (utils.get_item_properties(
                    s, columns,
                    formatters={},
                ) for s in data))


class SetCredential(command.Command):
    _description = _("Set credential properties")

    def get_parser(self, prog_name):
        parser = super(SetCredential, self).get_parser(prog_name)
        parser.add_argument(
            'credential',
            metavar='<credential-id>',
            help=_('ID of credential to change'),
        )
        parser.add_argument(
            '--user',
            metavar='<user>',
            required=True,
            help=_('User that owns the credential (name or ID)'),
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            required=True,
            help=_('New credential type: cert, ec2, totp and so on'),
        )
        parser.add_argument(
            '--data',
            metavar='<data>',
            required=True,
            help=_('New credential data'),
        )
        parser.add_argument(
            '--project',
            metavar='<project>',
            help=_('Project which limits the scope of '
                   'the credential (name or ID)'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity

        user_id = utils.find_resource(identity_client.users,
                                      parsed_args.user).id

        if parsed_args.project:
            project = utils.find_resource(identity_client.projects,
                                          parsed_args.project).id
        else:
            project = None

        identity_client.credentials.update(parsed_args.credential,
                                           user=user_id,
                                           type=parsed_args.type,
                                           blob=parsed_args.data,
                                           project=project)


class ShowCredential(command.ShowOne):
    _description = _("Display credential details")

    def get_parser(self, prog_name):
        parser = super(ShowCredential, self).get_parser(prog_name)
        parser.add_argument(
            'credential',
            metavar='<credential-id>',
            help=_('ID of credential to display'),
        )
        return parser

    def take_action(self, parsed_args):
        identity_client = self.app.client_manager.identity
        credential = utils.find_resource(identity_client.credentials,
                                         parsed_args.credential)

        credential._info.pop('links')
        return zip(*sorted(credential._info.items()))



#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock
from unittest.mock import call

from osc_lib.cli import format_columns
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.tests.unit.volume.v2 import fakes as volume_fakes
from openstackclient.volume.v2 import consistency_group


class TestConsistencyGroup(volume_fakes.TestVolume):

    def setUp(self):
        super(TestConsistencyGroup, self).setUp()

        # Get a shortcut to the TransferManager Mock
        self.consistencygroups_mock = (
            self.app.client_manager.volume.consistencygroups)
        self.consistencygroups_mock.reset_mock()

        self.cgsnapshots_mock = (
            self.app.client_manager.volume.cgsnapshots)
        self.cgsnapshots_mock.reset_mock()

        self.volumes_mock = (
            self.app.client_manager.volume.volumes)
        self.volumes_mock.reset_mock()

        self.types_mock = self.app.client_manager.volume.volume_types
        self.types_mock.reset_mock()


class TestConsistencyGroupAddVolume(TestConsistencyGroup):

    _consistency_group = (
        volume_fakes.FakeConsistencyGroup.create_one_consistency_group())

    def setUp(self):
        super(TestConsistencyGroupAddVolume, self).setUp()

        self.consistencygroups_mock.get.return_value = (
            self._consistency_group)
        # Get the command object to test
        self.cmd = \
            consistency_group.AddVolumeToConsistencyGroup(self.app, None)

    def test_add_one_volume_to_consistency_group(self):
        volume = volume_fakes.FakeVolume.create_one_volume()
        self.volumes_mock.get.return_value = volume
        arglist = [
            self._consistency_group.id,
            volume.id,
        ]
        verifylist = [
            ('consistency_group', self._consistency_group.id),
            ('volumes', [volume.id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'add_volumes': volume.id,
        }
        self.consistencygroups_mock.update.assert_called_once_with(
            self._consistency_group.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_add_multiple_volumes_to_consistency_group(self):
        volumes = volume_fakes.FakeVolume.create_volumes(count=2)
        self.volumes_mock.get = volume_fakes.FakeVolume.get_volumes(volumes)
        arglist = [
            self._consistency_group.id,
            volumes[0].id,
            volumes[1].id,
        ]
        verifylist = [
            ('consistency_group', self._consistency_group.id),
            ('volumes', [volumes[0].id, volumes[1].id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'add_volumes': volumes[0].id + ',' + volumes[1].id,
        }
        self.consistencygroups_mock.update.assert_called_once_with(
            self._consistency_group.id,
            **kwargs
        )
        self.assertIsNone(result)

    @mock.patch.object(consistency_group.LOG, 'error')
    def test_add_multiple_volumes_to_consistency_group_with_exception(
            self, mock_error):
        volume = volume_fakes.FakeVolume.create_one_volume()
        arglist = [
            self._consistency_group.id,
            volume.id,
            'unexist_volume',
        ]
        verifylist = [
            ('consistency_group', self._consistency_group.id),
            ('volumes', [volume.id, 'unexist_volume']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [volume,
                            exceptions.CommandError,
                            self._consistency_group]
        with mock.patch.object(utils, 'find_resource',
                               side_effect=find_mock_result) as find_mock:
            result = self.cmd.take_action(parsed_args)
            mock_error.assert_called_with("1 of 2 volumes failed to add.")
            self.assertIsNone(result)
            find_mock.assert_any_call(self.consistencygroups_mock,
                                      self._consistency_group.id)
            find_mock.assert_any_call(self.volumes_mock,
                                      volume.id)
            find_mock.assert_any_call(self.volumes_mock,
                                      'unexist_volume')
            self.assertEqual(3, find_mock.call_count)
            self.consistencygroups_mock.update.assert_called_once_with(
                self._consistency_group.id, add_volumes=volume.id
            )


class TestConsistencyGroupCreate(TestConsistencyGroup):

    volume_type = volume_fakes.FakeVolumeType.create_one_volume_type()
    new_consistency_group = (
        volume_fakes.FakeConsistencyGroup.create_one_consistency_group())
    consistency_group_snapshot = (
        volume_fakes.
        FakeConsistencyGroupSnapshot.
        create_one_consistency_group_snapshot()
    )

    columns = (
        'availability_zone',
        'created_at',
        'description',
        'id',
        'name',
        'status',
        'volume_types',
    )
    data = (
        new_consistency_group.availability_zone,
        new_consistency_group.created_at,
        new_consistency_group.description,
        new_consistency_group.id,
        new_consistency_group.name,
        new_consistency_group.status,
        new_consistency_group.volume_types,
    )

    def setUp(self):
        super(TestConsistencyGroupCreate, self).setUp()
        self.consistencygroups_mock.create.return_value = (
            self.new_consistency_group)
        self.consistencygroups_mock.create_from_src.return_value = (
            self.new_consistency_group)
        self.consistencygroups_mock.get.return_value = (
            self.new_consistency_group)
        self.types_mock.get.return_value = self.volume_type
        self.cgsnapshots_mock.get.return_value = (
            self.consistency_group_snapshot)

        # Get the command object to test
        self.cmd = consistency_group.CreateConsistencyGroup(self.app, None)

    def test_consistency_group_create(self):
        arglist = [
            '--volume-type', self.volume_type.id,
            '--description', self.new_consistency_group.description,
            '--availability-zone',
            self.new_consistency_group.availability_zone,
            self.new_consistency_group.name,
        ]
        verifylist = [
            ('volume_type', self.volume_type.id),
            ('description', self.new_consistency_group.description),
            ('availability_zone',
             self.new_consistency_group.availability_zone),
            ('name', self.new_consistency_group.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.types_mock.get.assert_called_once_with(
            self.volume_type.id)
        self.consistencygroups_mock.get.assert_not_called()
        self.consistencygroups_mock.create.assert_called_once_with(
            self.volume_type.id,
            name=self.new_consistency_group.name,
            description=self.new_consistency_group.description,
            availability_zone=self.new_consistency_group.availability_zone,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_consistency_group_create_without_name(self):
        arglist = [
            '--volume-type', self.volume_type.id,
            '--description', self.new_consistency_group.description,
            '--availability-zone',
            self.new_consistency_group.availability_zone,
        ]
        verifylist = [
            ('volume_type', self.volume_type.id),
            ('description', self.new_consistency_group.description),
            ('availability_zone',
             self.new_consistency_group.availability_zone),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.types_mock.get.assert_called_once_with(
            self.volume_type.id)
        self.consistencygroups_mock.get.assert_not_called()
        self.consistencygroups_mock.create.assert_called_once_with(
            self.volume_type.id,
            name=None,
            description=self.new_consistency_group.description,
            availability_zone=self.new_consistency_group.availability_zone,
        )

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_consistency_group_create_from_source(self):
        arglist = [
            '--consistency-group-source', self.new_consistency_group.id,
            '--description', self.new_consistency_group.description,
            self.new_consistency_group.name,
        ]
        verifylist = [
            ('consistency_group_source', self.new_consistency_group.id),
            ('description', self.new_consistency_group.description),
            ('name', self.new_consistency_group.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.types_mock.get.assert_not_called()
        self.consistencygroups_mock.get.assert_called_once_with(
            self.new_consistency_group.id)
        self.consistencygroups_mock.create_from_src.assert_called_with(
            None,
            self.new_consistency_group.id,
            name=self.new_consistency_group.name,
            description=self.new_consistency_group.description,
        )

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_consistency_group_create_from_snapshot(self):
        arglist = [
            '--consistency-group-snapshot', self.consistency_group_snapshot.id,
            '--description', self.new_consistency_group.description,
            self.new_consistency_group.name,
        ]
        verifylist = [
            ('consistency_group_snapshot', self.consistency_group_snapshot.id),
            ('description', self.new_consistency_group.description),
            ('name', self.new_consistency_group.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.types_mock.get.assert_not_called()
        self.cgsnapshots_mock.get.assert_called_once_with(
            self.consistency_group_snapshot.id)
        self.consistencygroups_mock.create_from_src.assert_called_with(
            self.consistency_group_snapshot.id,
            None,
            name=self.new_consistency_group.name,
            description=self.new_consistency_group.description,
        )

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestConsistencyGroupDelete(TestConsistencyGroup):

    consistency_groups =\
        volume_fakes.FakeConsistencyGroup.create_consistency_groups(count=2)

    def setUp(self):
        super(TestConsistencyGroupDelete, self).setUp()

        self.consistencygroups_mock.get = volume_fakes.FakeConsistencyGroup.\
            get_consistency_groups(self.consistency_groups)
        self.consistencygroups_mock.delete.return_value = None

        # Get the command object to mock
        self.cmd = consistency_group.DeleteConsistencyGroup(self.app, None)

    def test_consistency_group_delete(self):
        arglist = [
            self.consistency_groups[0].id
        ]
        verifylist = [
            ("consistency_groups", [self.consistency_groups[0].id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.consistencygroups_mock.delete.assert_called_with(
            self.consistency_groups[0].id, False)
        self.assertIsNone(result)

    def test_consistency_group_delete_with_force(self):
        arglist = [
            '--force',
            self.consistency_groups[0].id,
        ]
        verifylist = [
            ('force', True),
            ("consistency_groups", [self.consistency_groups[0].id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.consistencygroups_mock.delete.assert_called_with(
            self.consistency_groups[0].id, True)
        self.assertIsNone(result)

    def test_delete_multiple_consistency_groups(self):
        arglist = []
        for b in self.consistency_groups:
            arglist.append(b.id)
        verifylist = [
            ('consistency_groups', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        calls = []
        for b in self.consistency_groups:
            calls.append(call(b.id, False))
        self.consistencygroups_mock.delete.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_delete_multiple_consistency_groups_with_exception(self):
        arglist = [
            self.consistency_groups[0].id,
            'unexist_consistency_group',
        ]
        verifylist = [
            ('consistency_groups', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [self.consistency_groups[0],
                            exceptions.CommandError]
        with mock.patch.object(utils, 'find_resource',
                               side_effect=find_mock_result) as find_mock:
            try:
                self.cmd.take_action(parsed_args)
                self.fail('CommandError should be raised.')
            except exceptions.CommandError as e:
                self.assertEqual('1 of 2 consistency groups failed to delete.',
                                 str(e))

            find_mock.assert_any_call(self.consistencygroups_mock,
                                      self.consistency_groups[0].id)
            find_mock.assert_any_call(self.consistencygroups_mock,
                                      'unexist_consistency_group')

            self.assertEqual(2, find_mock.call_count)
            self.consistencygroups_mock.delete.assert_called_once_with(
                self.consistency_groups[0].id, False
            )


class TestConsistencyGroupList(TestConsistencyGroup):

    consistency_groups = (
        volume_fakes.FakeConsistencyGroup.create_consistency_groups(count=2))

    columns = [
        'ID',
        'Status',
        'Name',
    ]
    columns_long = [
        'ID',
        'Status',
        'Availability Zone',
        'Name',
        'Description',
        'Volume Types',
    ]
    data = []
    for c in consistency_groups:
        data.append((
            c.id,
            c.status,
            c.name,
        ))
    data_long = []
    for c in consistency_groups:
        data_long.append((
            c.id,
            c.status,
            c.availability_zone,
            c.name,
            c.description,
            format_columns.ListColumn(c.volume_types)
        ))

    def setUp(self):
        super(TestConsistencyGroupList, self).setUp()

        self.consistencygroups_mock.list.return_value = self.consistency_groups
        # Get the command to test
        self.cmd = consistency_group.ListConsistencyGroup(self.app, None)

    def test_consistency_group_list_without_options(self):
        arglist = []
        verifylist = [
            ("all_projects", False),
            ("long", False),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.consistencygroups_mock.list.assert_called_once_with(
            detailed=True, search_opts={'all_tenants': False})
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, list(data))

    def test_consistency_group_list_with_all_project(self):
        arglist = [
            "--all-projects"
        ]
        verifylist = [
            ("all_projects", True),
            ("long", False),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.consistencygroups_mock.list.assert_called_once_with(
            detailed=True, search_opts={'all_tenants': True})
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, list(data))

    def test_consistency_group_list_with_long(self):
        arglist = [
            "--long",
        ]
        verifylist = [
            ("all_projects", False),
            ("long", True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.consistencygroups_mock.list.assert_called_once_with(
            detailed=True, search_opts={'all_tenants': False})
        self.assertEqual(self.columns_long, columns)
        self.assertCountEqual(self.data_long, list(data))


class TestConsistencyGroupRemoveVolume(TestConsistencyGroup):

    _consistency_group = (
        volume_fakes.FakeConsistencyGroup.create_one_consistency_group())

    def setUp(self):
        super(TestConsistencyGroupRemoveVolume, self).setUp()

        self.consistencygroups_mock.get.return_value = (
            self._consistency_group)
        # Get the command object to test
        self.cmd = \
            consistency_group.RemoveVolumeFromConsistencyGroup(self.app, None)

    def test_remove_one_volume_from_consistency_group(self):
        volume = volume_fakes.FakeVolume.create_one_volume()
        self.volumes_mock.get.return_value = volume
        arglist = [
            self._consistency_group.id,
            volume.id,
        ]
        verifylist = [
            ('consistency_group', self._consistency_group.id),
            ('volumes', [volume.id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'remove_volumes': volume.id,
        }
        self.consistencygroups_mock.update.assert_called_once_with(
            self._consistency_group.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_remove_multi_volumes_from_consistency_group(self):
        volumes = volume_fakes.FakeVolume.create_volumes(count=2)
        self.volumes_mock.get = volume_fakes.FakeVolume.get_volumes(volumes)
        arglist = [
            self._consistency_group.id,
            volumes[0].id,
            volumes[1].id,
        ]
        verifylist = [
            ('consistency_group', self._consistency_group.id),
            ('volumes', [volumes[0].id, volumes[1].id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'remove_volumes': volumes[0].id + ',' + volumes[1].id,
        }
        self.consistencygroups_mock.update.assert_called_once_with(
            self._consistency_group.id,
            **kwargs
        )
        self.assertIsNone(result)

    @mock.patch.object(consistency_group.LOG, 'error')
    def test_remove_multiple_volumes_from_consistency_group_with_exception(
            self, mock_error):
        volume = volume_fakes.FakeVolume.create_one_volume()
        arglist = [
            self._consistency_group.id,
            volume.id,
            'unexist_volume',
        ]
        verifylist = [
            ('consistency_group', self._consistency_group.id),
            ('volumes', [volume.id, 'unexist_volume']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = [volume,
                            exceptions.CommandError,
                            self._consistency_group]
        with mock.patch.object(utils, 'find_resource',
                               side_effect=find_mock_result) as find_mock:
            result = self.cmd.take_action(parsed_args)
            mock_error.assert_called_with("1 of 2 volumes failed to remove.")
            self.assertIsNone(result)
            find_mock.assert_any_call(self.consistencygroups_mock,
                                      self._consistency_group.id)
            find_mock.assert_any_call(self.volumes_mock,
                                      volume.id)
            find_mock.assert_any_call(self.volumes_mock,
                                      'unexist_volume')
            self.assertEqual(3, find_mock.call_count)
            self.consistencygroups_mock.update.assert_called_once_with(
                self._consistency_group.id, remove_volumes=volume.id
            )


class TestConsistencyGroupSet(TestConsistencyGroup):

    consistency_group = (
        volume_fakes.FakeConsistencyGroup.create_one_consistency_group())

    def setUp(self):
        super(TestConsistencyGroupSet, self).setUp()

        self.consistencygroups_mock.get.return_value = (
            self.consistency_group)
        # Get the command object to test
        self.cmd = consistency_group.SetConsistencyGroup(self.app, None)

    def test_consistency_group_set_name(self):
        new_name = 'new_name'
        arglist = [
            '--name', new_name,
            self.consistency_group.id,
        ]
        verifylist = [
            ('name', new_name),
            ('description', None),
            ('consistency_group', self.consistency_group.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'name': new_name,
        }
        self.consistencygroups_mock.update.assert_called_once_with(
            self.consistency_group.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_consistency_group_set_description(self):
        new_description = 'new_description'
        arglist = [
            '--description', new_description,
            self.consistency_group.id,
        ]
        verifylist = [
            ('name', None),
            ('description', new_description),
            ('consistency_group', self.consistency_group.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'description': new_description,
        }
        self.consistencygroups_mock.update.assert_called_once_with(
            self.consistency_group.id,
            **kwargs
        )
        self.assertIsNone(result)


class TestConsistencyGroupShow(TestConsistencyGroup):
    columns = (
        'availability_zone',
        'created_at',
        'description',
        'id',
        'name',
        'status',
        'volume_types',
    )

    def setUp(self):
        super(TestConsistencyGroupShow, self).setUp()

        self.consistency_group = (
            volume_fakes.FakeConsistencyGroup.create_one_consistency_group())
        self.data = (
            self.consistency_group.availability_zone,
            self.consistency_group.created_at,
            self.consistency_group.description,
            self.consistency_group.id,
            self.consistency_group.name,
            self.consistency_group.status,
            self.consistency_group.volume_types,
        )
        self.consistencygroups_mock.get.return_value = self.consistency_group
        self.cmd = consistency_group.ShowConsistencyGroup(self.app, None)

    def test_consistency_group_show(self):
        arglist = [
            self.consistency_group.id
        ]
        verifylist = [
            ("consistency_group", self.consistency_group.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.consistencygroups_mock.get.assert_called_once_with(
            self.consistency_group.id)
        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)



#   Copyright 2014 CERN.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import copy
from unittest import mock

from osc_lib import exceptions

from openstackclient.identity.v3 import mapping
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes


class TestMapping(identity_fakes.TestFederatedIdentity):

    def setUp(self):
        super(TestMapping, self).setUp()

        federation_lib = self.app.client_manager.identity.federation
        self.mapping_mock = federation_lib.mappings
        self.mapping_mock.reset_mock()


class TestMappingCreate(TestMapping):

    def setUp(self):
        super(TestMappingCreate, self).setUp()
        self.mapping_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.MAPPING_RESPONSE),
            loaded=True
        )
        self.cmd = mapping.CreateMapping(self.app, None)

    def test_create_mapping(self):
        arglist = [
            '--rules', identity_fakes.mapping_rules_file_path,
            identity_fakes.mapping_id
        ]
        verifylist = [
            ('mapping', identity_fakes.mapping_id),
            ('rules', identity_fakes.mapping_rules_file_path)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        mocker = mock.Mock()
        mocker.return_value = identity_fakes.MAPPING_RULES
        with mock.patch("openstackclient.identity.v3.mapping."
                        "CreateMapping._read_rules", mocker):
            columns, data = self.cmd.take_action(parsed_args)

        self.mapping_mock.create.assert_called_with(
            mapping_id=identity_fakes.mapping_id,
            rules=identity_fakes.MAPPING_RULES)

        collist = ('id', 'rules')
        self.assertEqual(collist, columns)

        datalist = (identity_fakes.mapping_id,
                    identity_fakes.MAPPING_RULES)
        self.assertEqual(datalist, data)


class TestMappingDelete(TestMapping):

    def setUp(self):
        super(TestMappingDelete, self).setUp()
        self.mapping_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.MAPPING_RESPONSE),
            loaded=True)

        self.mapping_mock.delete.return_value = None
        self.cmd = mapping.DeleteMapping(self.app, None)

    def test_delete_mapping(self):
        arglist = [
            identity_fakes.mapping_id
        ]
        verifylist = [
            ('mapping', [identity_fakes.mapping_id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.mapping_mock.delete.assert_called_with(
            identity_fakes.mapping_id)
        self.assertIsNone(result)


class TestMappingList(TestMapping):

    def setUp(self):
        super(TestMappingList, self).setUp()
        self.mapping_mock.get.return_value = fakes.FakeResource(
            None,
            {'id': identity_fakes.mapping_id},
            loaded=True)
        # Pretend list command returns list of two mappings.
        # NOTE(marek-denis): We are returning FakeResources with mapping id
        # only as ShowMapping class is implemented in a way where rules will
        # not be displayed, only mapping ids.
        self.mapping_mock.list.return_value = [
            fakes.FakeResource(
                None,
                {'id': identity_fakes.mapping_id},
                loaded=True,
            ),
            fakes.FakeResource(
                None,
                {'id': 'extra_mapping'},
                loaded=True,
            ),
        ]

        # Get the command object to test
        self.cmd = mapping.ListMapping(self.app, None)

    def test_mapping_list(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.mapping_mock.list.assert_called_with()

        collist = ('ID',)
        self.assertEqual(collist, columns)

        datalist = [(identity_fakes.mapping_id,), ('extra_mapping',)]
        self.assertEqual(datalist, data)


class TestMappingSet(TestMapping):

    def setUp(self):
        super(TestMappingSet, self).setUp()

        self.mapping_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.MAPPING_RESPONSE),
            loaded=True
        )

        self.mapping_mock.update.return_value = fakes.FakeResource(
            None,
            identity_fakes.MAPPING_RESPONSE_2,
            loaded=True
        )

        # Get the command object to test
        self.cmd = mapping.SetMapping(self.app, None)

    def test_set_new_rules(self):
        arglist = [
            '--rules', identity_fakes.mapping_rules_file_path,
            identity_fakes.mapping_id
        ]
        verifylist = [
            ('mapping', identity_fakes.mapping_id),
            ('rules', identity_fakes.mapping_rules_file_path)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        mocker = mock.Mock()
        mocker.return_value = identity_fakes.MAPPING_RULES_2
        with mock.patch("openstackclient.identity.v3.mapping."
                        "SetMapping._read_rules", mocker):
            result = self.cmd.take_action(parsed_args)
        self.mapping_mock.update.assert_called_with(
            mapping=identity_fakes.mapping_id,
            rules=identity_fakes.MAPPING_RULES_2)

        self.assertIsNone(result)

    def test_set_rules_wrong_file_path(self):
        arglist = [
            '--rules', identity_fakes.mapping_rules_file_path,
            identity_fakes.mapping_id
        ]
        verifylist = [
            ('mapping', identity_fakes.mapping_id),
            ('rules', identity_fakes.mapping_rules_file_path)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestMappingShow(TestMapping):

    def setUp(self):
        super(TestMappingShow, self).setUp()

        self.mapping_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.MAPPING_RESPONSE),
            loaded=True
        )

        self.cmd = mapping.ShowMapping(self.app, None)

    def test_mapping_show(self):
        arglist = [
            identity_fakes.mapping_id
        ]
        verifylist = [
            ('mapping', identity_fakes.mapping_id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.mapping_mock.get.assert_called_with(
            identity_fakes.mapping_id)

        collist = ('id', 'rules')
        self.assertEqual(collist, columns)

        datalist = (identity_fakes.mapping_id,
                    identity_fakes.MAPPING_RULES)
        self.assertEqual(datalist, data)



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Host action implementations"""

from osc_lib.command import command
from osc_lib import utils

from openstackclient.i18n import _


class ListHost(command.Lister):
    _description = _("List hosts")

    def get_parser(self, prog_name):
        parser = super(ListHost, self).get_parser(prog_name)
        parser.add_argument(
            "--zone",
            metavar="<zone>",
            help=_("Only return hosts in the availability zone")
        )
        return parser

    def take_action(self, parsed_args):
        compute_client = self.app.client_manager.compute
        columns = (
            "Host Name",
            "Service",
            "Zone"
        )
        data = compute_client.api.host_list(parsed_args.zone)
        return (columns,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data))


class SetHost(command.Command):
    _description = _("Set host properties")

    def get_parser(self, prog_name):
        parser = super(SetHost, self).get_parser(prog_name)
        parser.add_argument(
            "host",
            metavar="<host>",
            help=_("Host to modify (name only)")
        )
        status = parser.add_mutually_exclusive_group()
        status.add_argument(
            '--enable',
            action='store_true',
            help=_("Enable the host")
        )
        status.add_argument(
            '--disable',
            action='store_true',
            help=_("Disable the host")
        )
        maintenance = parser.add_mutually_exclusive_group()
        maintenance.add_argument(
            '--enable-maintenance',
            action='store_true',
            help=_("Enable maintenance mode for the host")
        )
        maintenance.add_argument(
            '--disable-maintenance',
            action='store_true',
            help=_("Disable maintenance mode for the host")
        )
        return parser

    def take_action(self, parsed_args):
        kwargs = {}

        if parsed_args.enable:
            kwargs['status'] = 'enable'
        if parsed_args.disable:
            kwargs['status'] = 'disable'
        if parsed_args.enable_maintenance:
            kwargs['maintenance_mode'] = 'enable'
        if parsed_args.disable_maintenance:
            kwargs['maintenance_mode'] = 'disable'

        compute_client = self.app.client_manager.compute

        compute_client.api.host_set(
            parsed_args.host,
            **kwargs
        )


class ShowHost(command.Lister):
    _description = _("Display host details")

    def get_parser(self, prog_name):
        parser = super(ShowHost, self).get_parser(prog_name)
        parser.add_argument(
            "host",
            metavar="<host>",
            help=_("Name of host")
        )
        return parser

    def take_action(self, parsed_args):
        compute_client = self.app.client_manager.compute
        columns = (
            "Host",
            "Project",
            "CPU",
            "Memory MB",
            "Disk GB"
        )

        data = compute_client.api.host_show(parsed_args.host)

        return (columns,
                (utils.get_dict_properties(
                    s, columns,
                ) for s in data))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import tempfile
import uuid

from openstackclient.tests.functional.object.v1 import common

BASIC_LIST_HEADERS = ['Name']
CONTAINER_FIELDS = ['account', 'container', 'x-trans-id']
OBJECT_FIELDS = ['object', 'container', 'etag']


class ObjectTests(common.ObjectStoreTests):
    """Functional tests for Object Store object commands"""

    CONTAINER_NAME = uuid.uuid4().hex

    def setUp(self):
        super(ObjectTests, self).setUp()
        # Skip tests if no object-store is present
        if not self.haz_object_store:
            self.skipTest("No object-store service present")

    def test_object(self):
        with tempfile.NamedTemporaryFile(mode='w+') as f:
            f.write('test content')
            f.flush()
            self._test_object(f.name)

    def _test_object(self, object_file):
        raw_output = self.openstack('container create ' + self.CONTAINER_NAME)
        items = self.parse_listing(raw_output)
        self.assert_show_fields(items, CONTAINER_FIELDS)

        raw_output = self.openstack('container list')
        items = self.parse_listing(raw_output)
        self.assert_table_structure(items, BASIC_LIST_HEADERS)

        self.openstack('container show ' + self.CONTAINER_NAME)
        # TODO(stevemar): Assert returned fields

        self.openstack('container save ' + self.CONTAINER_NAME)
        # TODO(stevemar): Assert returned fields

        raw_output = self.openstack('object create %s %s' %
                                    (self.CONTAINER_NAME, object_file))
        items = self.parse_listing(raw_output)
        self.assert_show_fields(items, OBJECT_FIELDS)

        raw_output = self.openstack('object list %s' % self.CONTAINER_NAME)
        items = self.parse_listing(raw_output)
        self.assert_table_structure(items, BASIC_LIST_HEADERS)

        self.openstack('object save %s %s' %
                       (self.CONTAINER_NAME, object_file))
        # TODO(stevemar): Assert returned fields

        tmp_file = 'tmp.txt'
        self.addCleanup(os.remove, tmp_file)
        self.openstack('object save %s %s --file %s' %
                       (self.CONTAINER_NAME, object_file, tmp_file))
        # TODO(stevemar): Assert returned fields

        raw_output = self.openstack('object save %s %s --file -' %
                                    (self.CONTAINER_NAME, object_file))
        self.assertEqual(raw_output, 'test content')

        self.openstack('object show %s %s' %
                       (self.CONTAINER_NAME, object_file))
        # TODO(stevemar): Assert returned fields

        raw_output = self.openstack('object delete %s %s' %
                                    (self.CONTAINER_NAME, object_file))
        self.assertEqual(0, len(raw_output))

        self.openstack('object create %s %s' %
                       (self.CONTAINER_NAME, object_file))
        raw_output = self.openstack('container delete -r %s' %
                                    self.CONTAINER_NAME)
        self.assertEqual(0, len(raw_output))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstackclient.tests.functional.identity.v2 import common


class RoleTests(common.IdentityTests):

    def test_role_create(self):
        self._create_dummy_role()

    def test_role_delete(self):
        role_name = self._create_dummy_role(add_clean_up=False)
        raw_output = self.openstack('role delete %s' % role_name)
        self.assertEqual(0, len(raw_output))

    def test_role_list(self):
        self._create_dummy_role()
        raw_output = self.openstack('role list')
        items = self.parse_listing(raw_output)
        self.assert_table_structure(items, common.BASIC_LIST_HEADERS)

    def test_role_show(self):
        role_name = self._create_dummy_role()
        raw_output = self.openstack('role show %s' % role_name)
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.ROLE_FIELDS)

    def test_role_add(self):
        role_name = self._create_dummy_role()
        username = self._create_dummy_user()
        raw_output = self.openstack(
            'role add '
            '--project %(project)s '
            '--user %(user)s '
            '%(role)s' % {'project': self.project_name,
                          'user': username,
                          'role': role_name})
        self.addCleanup(
            self.openstack,
            'role remove '
            '--project %(project)s '
            '--user %(user)s '
            '%(role)s' % {'project': self.project_name,
                          'user': username,
                          'role': role_name})
        items = self.parse_show(raw_output)
        self.assert_show_fields(items, self.ROLE_FIELDS)

    def test_role_remove(self):
        role_name = self._create_dummy_role()
        username = self._create_dummy_user()
        add_raw_output = self.openstack(
            'role add '
            '--project %(project)s '
            '--user %(user)s '
            '%(role)s' % {'project': self.project_name,
                          'user': username,
                          'role': role_name})
        del_raw_output = self.openstack(
            'role remove '
            '--project %(project)s '
            '--user %(user)s '
            '%(role)s' % {'project': self.project_name,
                          'user': username,
                          'role': role_name})
        items = self.parse_show(add_raw_output)
        self.assert_show_fields(items, self.ROLE_FIELDS)
        self.assertEqual(0, len(del_raw_output))



#   Copyright 2012-2013 OpenStack Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Hypervisor action implementations"""

import json
import re

from novaclient import api_versions
from novaclient import exceptions as nova_exceptions
from osc_lib.cli import format_columns
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from openstackclient.i18n import _


class ListHypervisor(command.Lister):
    _description = _("List hypervisors")

    def get_parser(self, prog_name):
        parser = super(ListHypervisor, self).get_parser(prog_name)
        parser.add_argument(
            '--matching',
            metavar='<hostname>',
            help=_("Filter hypervisors using <hostname> substring")
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_(
                "The UUID of the last hypervisor of the previous page; "
                "displays list of hypervisors after 'marker'. "
                "(supported with --os-compute-api-version 2.33 or above)"
            ),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_(
                "Maximum number of hypervisors to display. Note that there "
                "is a configurable max limit on the server, and the limit "
                "that is used will be the minimum of what is requested "
                "here and what is configured in the server. "
                "(supported with --os-compute-api-version 2.33 or above)"
            ),
        )
        parser.add_argument(
            '--long',
            action='store_true',
            help=_("List additional fields in output")
        )
        return parser

    def take_action(self, parsed_args):
        compute_client = self.app.client_manager.compute

        list_opts = {}

        if parsed_args.matching and (parsed_args.marker or parsed_args.limit):
            msg = _(
                '--matching is not compatible with --marker or --limit'
            )
            raise exceptions.CommandError(msg)

        if parsed_args.marker:
            if compute_client.api_version < api_versions.APIVersion('2.33'):
                msg = _(
                    '--os-compute-api-version 2.33 or greater is required to '
                    'support the --marker option'
                )
                raise exceptions.CommandError(msg)
            list_opts['marker'] = parsed_args.marker

        if parsed_args.limit:
            if compute_client.api_version < api_versions.APIVersion('2.33'):
                msg = _(
                    '--os-compute-api-version 2.33 or greater is required to '
                    'support the --limit option'
                )
                raise exceptions.CommandError(msg)
            list_opts['limit'] = parsed_args.limit

        columns = (
            "ID",
            "Hypervisor Hostname",
            "Hypervisor Type",
            "Host IP",
            "State"
        )
        if parsed_args.long:
            columns += ("vCPUs Used", "vCPUs", "Memory MB Used", "Memory MB")

        if parsed_args.matching:
            data = compute_client.hypervisors.search(parsed_args.matching)
        else:
            data = compute_client.hypervisors.list(**list_opts)

        return (
            columns,
            (utils.get_item_properties(s, columns) for s in data),
        )


class ShowHypervisor(command.ShowOne):
    _description = _("Display hypervisor details")

    def get_parser(self, prog_name):
        parser = super(ShowHypervisor, self).get_parser(prog_name)
        parser.add_argument(
            "hypervisor",
            metavar="<hypervisor>",
            help=_("Hypervisor to display (name or ID)")
        )
        return parser

    def take_action(self, parsed_args):
        compute_client = self.app.client_manager.compute
        hypervisor = utils.find_resource(compute_client.hypervisors,
                                         parsed_args.hypervisor)._info.copy()

        aggregates = compute_client.aggregates.list()
        hypervisor["aggregates"] = list()
        if aggregates:
            # Hypervisors in nova cells are prefixed by "<cell>@"
            if "@" in hypervisor['service']['host']:
                cell, service_host = hypervisor['service']['host'].split(
                    '@', 1)
            else:
                cell = None
                service_host = hypervisor['service']['host']

            if cell:
                # The host aggregates are also prefixed by "<cell>@"
                member_of = [aggregate.name
                             for aggregate in aggregates
                             if cell in aggregate.name and
                             service_host in aggregate.hosts]
            else:
                member_of = [aggregate.name
                             for aggregate in aggregates
                             if service_host in aggregate.hosts]
            hypervisor["aggregates"] = member_of

        try:
            uptime = compute_client.hypervisors.uptime(hypervisor['id'])._info
            # Extract data from uptime value
            # format: 0 up 0,  0 users,  load average: 0, 0, 0
            # example: 17:37:14 up  2:33,  3 users,
            #          load average: 0.33, 0.36, 0.34
            m = re.match(
                r"\s*(.+)\sup\s+(.+),\s+(.+)\susers?,\s+load average:\s(.+)",
                uptime['uptime'])
            if m:
                hypervisor["host_time"] = m.group(1)
                hypervisor["uptime"] = m.group(2)
                hypervisor["users"] = m.group(3)
                hypervisor["load_average"] = m.group(4)
        except nova_exceptions.HTTPNotImplemented:
            pass

        hypervisor["service_id"] = hypervisor["service"]["id"]
        hypervisor["service_host"] = hypervisor["service"]["host"]
        del hypervisor["service"]

        if compute_client.api_version < api_versions.APIVersion('2.28'):
            # microversion 2.28 transformed this to a JSON blob rather than a
            # string; on earlier fields, do this manually
            if hypervisor['cpu_info']:
                hypervisor['cpu_info'] = json.loads(hypervisor['cpu_info'])
            else:
                hypervisor['cpu_info'] = {}

        columns = tuple(sorted(hypervisor))
        data = utils.get_dict_properties(
            hypervisor, columns,
            formatters={
                'cpu_info': format_columns.DictColumn,
            })

        return (columns, data)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
from unittest import mock

from osc_lib.cli import format_columns

from openstackclient.image.v1 import image
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.image.v1 import fakes as image_fakes


class TestImage(image_fakes.TestImagev1):

    pass


class TestImageCreate(TestImage):

    new_image = image_fakes.create_one_image()
    columns = (
        'container_format',
        'disk_format',
        'id',
        'is_public',
        'min_disk',
        'min_ram',
        'name',
        'owner',
        'properties',
        'protected',
        'size'
    )
    data = (
        new_image.container_format,
        new_image.disk_format,
        new_image.id,
        new_image.is_public,
        new_image.min_disk,
        new_image.min_ram,
        new_image.name,
        new_image.owner_id,
        format_columns.DictColumn(new_image.properties),
        new_image.is_protected,
        new_image.size
    )

    def setUp(self):
        super(TestImageCreate, self).setUp()

        self.client.create_image = mock.Mock(return_value=self.new_image)
        self.client.find_image = mock.Mock(return_value=self.new_image)
        self.client.update_image = mock.Mock(return_image=self.new_image)

        # Get the command object to test
        self.cmd = image.CreateImage(self.app, None)

    @mock.patch('sys.stdin', side_effect=[None])
    def test_image_reserve_no_options(self, raw_input):
        arglist = [
            self.new_image.name,
        ]
        verifylist = [
            ('container_format', image.DEFAULT_CONTAINER_FORMAT),
            ('disk_format', image.DEFAULT_DISK_FORMAT),
            ('name', self.new_image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        # ImageManager.create(name=, **)
        self.client.create_image.assert_called_with(
            name=self.new_image.name,
            container_format=image.DEFAULT_CONTAINER_FORMAT,
            disk_format=image.DEFAULT_DISK_FORMAT
        )

        # Verify update() was not called, if it was show the args
        self.assertEqual(self.client.update_image.call_args_list, [])

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    @mock.patch('sys.stdin', side_effect=[None])
    def test_image_reserve_options(self, raw_input):
        arglist = [
            '--container-format', 'ovf',
            '--disk-format', 'ami',
            '--min-disk', '10',
            '--min-ram', '4',
            '--protected',
            '--private',
            '--project', 'q',
            self.new_image.name,
        ]
        verifylist = [
            ('container_format', 'ovf'),
            ('disk_format', 'ami'),
            ('min_disk', 10),
            ('min_ram', 4),
            ('protected', True),
            ('unprotected', False),
            ('public', False),
            ('private', True),
            ('project', 'q'),
            ('name', self.new_image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        # ImageManager.create(name=, **)
        self.client.create_image.assert_called_with(
            name=self.new_image.name,
            container_format='ovf',
            disk_format='ami',
            min_disk=10,
            min_ram=4,
            is_protected=True,
            is_public=False,
            owner_id='q',
        )

        # Verify update() was not called, if it was show the args
        self.assertEqual(self.client.update_image.call_args_list, [])

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    @mock.patch('openstackclient.image.v1.image.io.open', name='Open')
    def test_image_create_file(self, mock_open):
        mock_file = mock.Mock(name='File')
        mock_open.return_value = mock_file
        mock_open.read.return_value = self.data

        arglist = [
            '--file', 'filer',
            '--unprotected',
            '--public',
            '--property', 'Alpha=1',
            '--property', 'Beta=2',
            self.new_image.name,
        ]
        verifylist = [
            ('file', 'filer'),
            ('protected', False),
            ('unprotected', True),
            ('public', True),
            ('private', False),
            ('properties', {'Alpha': '1', 'Beta': '2'}),
            ('name', self.new_image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        # Ensure input file is opened
        mock_open.assert_called_with('filer', 'rb')

        # Ensure the input file is closed
        mock_file.close.assert_called_with()

        # ImageManager.create(name=, **)
        self.client.create_image.assert_called_with(
            name=self.new_image.name,
            container_format=image.DEFAULT_CONTAINER_FORMAT,
            disk_format=image.DEFAULT_DISK_FORMAT,
            is_protected=False,
            is_public=True,
            properties={
                'Alpha': '1',
                'Beta': '2',
            },
            data=mock_file,
        )

        # Verify update() was not called, if it was show the args
        self.assertEqual(self.client.update_image.call_args_list, [])

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestImageDelete(TestImage):

    _image = image_fakes.create_one_image()

    def setUp(self):
        super(TestImageDelete, self).setUp()

        # This is the return value for utils.find_resource()
        self.client.find_image = mock.Mock(return_value=self._image)
        self.client.delete_image = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = image.DeleteImage(self.app, None)

    def test_image_delete_no_options(self):
        arglist = [
            self._image.id,
        ]
        verifylist = [
            ('images', [self._image.id]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.client.delete_image.assert_called_with(self._image.id)
        self.assertIsNone(result)


class TestImageList(TestImage):

    _image = image_fakes.create_one_image()

    columns = (
        'ID',
        'Name',
        'Status',
    )
    datalist = (
        (
            _image.id,
            _image.name,
            _image.status
        ),
    )

    # create a image_info as the side_effect of the fake image_list()
    info = {
        'id': _image.id,
        'name': _image.name,
        'owner': _image.owner_id,
        'container_format': _image.container_format,
        'disk_format': _image.disk_format,
        'min_disk': _image.min_disk,
        'min_ram': _image.min_ram,
        'is_public': _image.is_public,
        'protected': _image.is_protected,
        'properties': _image.properties,
    }
    image_info = copy.deepcopy(info)

    def setUp(self):
        super(TestImageList, self).setUp()

        self.client.images = mock.Mock()
        self.client.images.side_effect = [
            [self._image], [],
        ]

        # Get the command object to test
        self.cmd = image.ListImage(self.app, None)

    def test_image_list_no_options(self):
        arglist = []
        verifylist = [
            ('public', False),
            ('private', False),
            ('long', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.images.assert_called_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    def test_image_list_public_option(self):
        arglist = [
            '--public',
        ]
        verifylist = [
            ('public', True),
            ('private', False),
            ('long', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.images.assert_called_with(
            is_public=True,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    def test_image_list_private_option(self):
        arglist = [
            '--private',
        ]
        verifylist = [
            ('public', False),
            ('private', True),
            ('long', False),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.images.assert_called_with(
            is_private=True,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    def test_image_list_long_option(self):
        arglist = [
            '--long',
        ]
        verifylist = [
            ('long', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.images.assert_called_with()

        collist = (
            'ID',
            'Name',
            'Disk Format',
            'Container Format',
            'Size',
            'Checksum',
            'Status',
            'Visibility',
            'Protected',
            'Project',
            'Properties',
        )

        self.assertEqual(collist, columns)
        datalist = ((
            self._image.id,
            self._image.name,
            self._image.disk_format,
            self._image.container_format,
            self._image.size,
            self._image.checksum,
            self._image.status,
            image.VisibilityColumn(self._image.is_public),
            self._image.is_protected,
            self._image.owner_id,
            format_columns.DictColumn(
                {'Alpha': 'a', 'Beta': 'b', 'Gamma': 'g'}),
        ), )
        self.assertCountEqual(datalist, tuple(data))

    @mock.patch('osc_lib.api.utils.simple_filter')
    def test_image_list_property_option(self, sf_mock):
        sf_mock.side_effect = [
            [self.image_info], [],
        ]

        arglist = [
            '--property', 'a=1',
        ]
        verifylist = [
            ('property', {'a': '1'}),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.images.assert_called_with()
        sf_mock.assert_called_with(
            [self._image],
            attr='a',
            value='1',
            property_field='properties',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    @mock.patch('osc_lib.utils.sort_items')
    def test_image_list_sort_option(self, si_mock):
        si_mock.side_effect = [
            [self._image], [],
        ]

        arglist = ['--sort', 'name:asc']
        verifylist = [('sort', 'name:asc')]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.images.assert_called_with()
        si_mock.assert_called_with(
            [self._image],
            'name:asc'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestImageSet(TestImage):

    _image = image_fakes.create_one_image()

    def setUp(self):
        super(TestImageSet, self).setUp()

        # This is the return value for utils.find_resource()
        self.client.find_image = mock.Mock(return_value=self._image)
        self.client.update_image = mock.Mock(return_value=self._image)

        # Get the command object to test
        self.cmd = image.SetImage(self.app, None)

    def test_image_set_no_options(self):
        arglist = [
            self._image.name,
        ]
        verifylist = [
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.client.update_image.assert_called_with(self._image.id, **{})
        self.assertIsNone(result)

    def test_image_set_options(self):
        arglist = [
            '--name', 'new-name',
            '--min-disk', '2',
            '--min-ram', '4',
            '--container-format', 'ovf',
            '--disk-format', 'vmdk',
            '--size', '35165824',
            '--project', 'new-owner',
            self._image.name,
        ]
        verifylist = [
            ('name', 'new-name'),
            ('min_disk', 2),
            ('min_ram', 4),
            ('container_format', 'ovf'),
            ('disk_format', 'vmdk'),
            ('size', 35165824),
            ('project', 'new-owner'),
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        kwargs = {
            'name': 'new-name',
            'owner': 'new-owner',
            'min_disk': 2,
            'min_ram': 4,
            'container_format': 'ovf',
            'disk_format': 'vmdk',
            'size': 35165824
        }
        # ImageManager.update(image, **kwargs)
        self.client.update_image.assert_called_with(
            self._image.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_image_set_bools1(self):
        arglist = [
            '--protected',
            '--private',
            self._image.name,
        ]
        verifylist = [
            ('protected', True),
            ('unprotected', False),
            ('public', False),
            ('private', True),
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        kwargs = {
            'is_protected': True,
            'is_public': False,
        }
        # ImageManager.update(image, **kwargs)
        self.client.update_image.assert_called_with(
            self._image.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_image_set_bools2(self):
        arglist = [
            '--unprotected',
            '--public',
            self._image.name,
        ]
        verifylist = [
            ('protected', False),
            ('unprotected', True),
            ('public', True),
            ('private', False),
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        kwargs = {
            'is_protected': False,
            'is_public': True,
        }
        # ImageManager.update(image, **kwargs)
        self.client.update_image.assert_called_with(
            self._image.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_image_set_properties(self):
        arglist = [
            '--property', 'Alpha=1',
            '--property', 'Beta=2',
            self._image.name,
        ]
        verifylist = [
            ('properties', {'Alpha': '1', 'Beta': '2'}),
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        kwargs = {
            'properties': {
                'Alpha': '1',
                'Beta': '2',
                'Gamma': 'g',
            },
        }
        # ImageManager.update(image, **kwargs)
        self.client.update_image.assert_called_with(
            self._image.id,
            **kwargs
        )
        self.assertIsNone(result)

    def test_image_update_volume(self):
        # Set up VolumeManager Mock
        volumes_mock = self.app.client_manager.volume.volumes
        volumes_mock.reset_mock()
        volumes_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy({'id': 'vol1', 'name': 'volly'}),
            loaded=True,
        )
        response = {
            "id": 'volume_id',
            "updated_at": 'updated_at',
            "status": 'uploading',
            "display_description": 'desc',
            "size": 'size',
            "volume_type": 'volume_type',
            "container_format": image.DEFAULT_CONTAINER_FORMAT,
            "disk_format": image.DEFAULT_DISK_FORMAT,
            "image": self._image.name,
        }
        full_response = {"os-volume_upload_image": response}
        volumes_mock.upload_to_image.return_value = (201, full_response)

        arglist = [
            '--volume', 'volly',
            '--name', 'updated_image',
            self._image.name,
        ]
        verifylist = [
            ('private', False),
            ('protected', False),
            ('public', False),
            ('unprotected', False),
            ('volume', 'volly'),
            ('force', False),
            ('name', 'updated_image'),
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        # VolumeManager.upload_to_image(volume, force, image_name,
        #     container_format, disk_format)
        volumes_mock.upload_to_image.assert_called_with(
            'vol1',
            False,
            self._image.name,
            '',
            '',
        )
        # ImageManager.update(image_id, remove_props=, **)
        self.client.update_image.assert_called_with(
            self._image.id,
            name='updated_image',
            volume='volly',
        )
        self.assertIsNone(result)

    def test_image_set_numeric_options_to_zero(self):
        arglist = [
            '--min-disk', '0',
            '--min-ram', '0',
            self._image.name,
        ]
        verifylist = [
            ('min_disk', 0),
            ('min_ram', 0),
            ('image', self._image.name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        kwargs = {
            'min_disk': 0,
            'min_ram': 0,
        }
        # ImageManager.update(image, **kwargs)
        self.client.update_image.assert_called_with(
            self._image.id,
            **kwargs
        )
        self.assertIsNone(result)


class TestImageShow(TestImage):

    _image = image_fakes.create_one_image(attrs={'size': 2000})
    columns = (
        'container_format',
        'disk_format',
        'id',
        'is_public',
        'min_disk',
        'min_ram',
        'name',
        'owner',
        'properties',
        'protected',
        'size',
    )
    data = (
        _image.container_format,
        _image.disk_format,
        _image.id,
        _image.is_public,
        _image.min_disk,
        _image.min_ram,
        _image.name,
        _image.owner_id,
        format_columns.DictColumn(_image.properties),
        _image.is_protected,
        _image.size,
    )

    def setUp(self):
        super(TestImageShow, self).setUp()

        self.client.find_image = mock.Mock(return_value=self._image)

        # Get the command object to test
        self.cmd = image.ShowImage(self.app, None)

    def test_image_show(self):
        arglist = [
            self._image.id,
        ]
        verifylist = [
            ('image', self._image.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_image.assert_called_with(
            self._image.id,
        )

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_image_show_human_readable(self):
        arglist = [
            '--human-readable',
            self._image.id,
        ]
        verifylist = [
            ('human_readable', True),
            ('image', self._image.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_image.assert_called_with(
            self._image.id,
        )

        size_index = columns.index('size')
        self.assertEqual(data[size_index].human_readable(), '2K')



#   Copyright 2018 SUSE Linux GmbH
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
import json
from unittest import mock

from osc_lib import exceptions
from osc_lib import utils

from openstackclient.identity.v3 import application_credential
from openstackclient.tests.unit import fakes
from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes


class TestApplicationCredential(identity_fakes.TestIdentityv3):

    def setUp(self):
        super(TestApplicationCredential, self).setUp()

        identity_manager = self.app.client_manager.identity
        self.app_creds_mock = identity_manager.application_credentials
        self.app_creds_mock.reset_mock()
        self.roles_mock = identity_manager.roles
        self.roles_mock.reset_mock()


class TestApplicationCredentialCreate(TestApplicationCredential):

    def setUp(self):
        super(TestApplicationCredentialCreate, self).setUp()

        self.roles_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.ROLE),
            loaded=True,
        )

        # Get the command object to test
        self.cmd = application_credential.CreateApplicationCredential(
            self.app, None)

    def test_application_credential_create_basic(self):
        self.app_creds_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.APP_CRED_BASIC),
            loaded=True,
        )

        name = identity_fakes.app_cred_name
        arglist = [
            name
        ]
        verifylist = [
            ('name', identity_fakes.app_cred_name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'secret': None,
            'roles': [],
            'expires_at': None,
            'description': None,
            'unrestricted': False,
            'access_rules': None,
        }
        self.app_creds_mock.create.assert_called_with(
            name,
            **kwargs
        )

        collist = ('access_rules', 'description', 'expires_at', 'id', 'name',
                   'project_id', 'roles', 'secret', 'unrestricted')
        self.assertEqual(collist, columns)
        datalist = (
            None,
            None,
            None,
            identity_fakes.app_cred_id,
            identity_fakes.app_cred_name,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.app_cred_secret,
            False,
        )
        self.assertEqual(datalist, data)

    def test_application_credential_create_with_options(self):
        name = identity_fakes.app_cred_name
        self.app_creds_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.APP_CRED_OPTIONS),
            loaded=True,
        )

        arglist = [
            name,
            '--secret', 'moresecuresecret',
            '--role', identity_fakes.role_id,
            '--expiration', identity_fakes.app_cred_expires_str,
            '--description', 'credential for testing'
        ]
        verifylist = [
            ('name', identity_fakes.app_cred_name),
            ('secret', 'moresecuresecret'),
            ('role', [identity_fakes.role_id]),
            ('expiration', identity_fakes.app_cred_expires_str),
            ('description', 'credential for testing')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'secret': 'moresecuresecret',
            'roles': [identity_fakes.role_id],
            'expires_at': identity_fakes.app_cred_expires,
            'description': 'credential for testing',
            'unrestricted': False,
            'access_rules': None,
        }
        self.app_creds_mock.create.assert_called_with(
            name,
            **kwargs
        )

        collist = ('access_rules', 'description', 'expires_at', 'id', 'name',
                   'project_id', 'roles', 'secret', 'unrestricted')
        self.assertEqual(collist, columns)
        datalist = (
            None,
            identity_fakes.app_cred_description,
            identity_fakes.app_cred_expires_str,
            identity_fakes.app_cred_id,
            identity_fakes.app_cred_name,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.app_cred_secret,
            False,
        )
        self.assertEqual(datalist, data)

    def test_application_credential_create_with_access_rules_string(self):
        name = identity_fakes.app_cred_name
        self.app_creds_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.APP_CRED_ACCESS_RULES),
            loaded=True,
        )

        arglist = [
            name,
            '--access-rules', identity_fakes.app_cred_access_rules,
        ]
        verifylist = [
            ('name', identity_fakes.app_cred_name),
            ('access_rules', identity_fakes.app_cred_access_rules),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'secret': None,
            'roles': [],
            'expires_at': None,
            'description': None,
            'unrestricted': False,
            'access_rules': json.loads(identity_fakes.app_cred_access_rules)
        }
        self.app_creds_mock.create.assert_called_with(
            name,
            **kwargs
        )

        collist = ('access_rules', 'description', 'expires_at', 'id', 'name',
                   'project_id', 'roles', 'secret', 'unrestricted')
        self.assertEqual(collist, columns)
        datalist = (
            identity_fakes.app_cred_access_rules,
            None,
            None,
            identity_fakes.app_cred_id,
            identity_fakes.app_cred_name,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.app_cred_secret,
            False,
        )
        self.assertEqual(datalist, data)

    @mock.patch('openstackclient.identity.v3.application_credential.json.load')
    @mock.patch('openstackclient.identity.v3.application_credential.open')
    def test_application_credential_create_with_access_rules_file(
            self, _, mock_json_load):
        mock_json_load.return_value = identity_fakes.app_cred_access_rules

        name = identity_fakes.app_cred_name
        self.app_creds_mock.create.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.APP_CRED_ACCESS_RULES),
            loaded=True,
        )

        arglist = [
            name,
            '--access-rules', identity_fakes.app_cred_access_rules_path,
        ]
        verifylist = [
            ('name', identity_fakes.app_cred_name),
            ('access_rules', identity_fakes.app_cred_access_rules_path),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        # Set expected values
        kwargs = {
            'secret': None,
            'roles': [],
            'expires_at': None,
            'description': None,
            'unrestricted': False,
            'access_rules': identity_fakes.app_cred_access_rules
        }
        self.app_creds_mock.create.assert_called_with(
            name,
            **kwargs
        )

        collist = ('access_rules', 'description', 'expires_at', 'id', 'name',
                   'project_id', 'roles', 'secret', 'unrestricted')
        self.assertEqual(collist, columns)
        datalist = (
            identity_fakes.app_cred_access_rules,
            None,
            None,
            identity_fakes.app_cred_id,
            identity_fakes.app_cred_name,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.app_cred_secret,
            False,
        )
        self.assertEqual(datalist, data)


class TestApplicationCredentialDelete(TestApplicationCredential):

    def setUp(self):
        super(TestApplicationCredentialDelete, self).setUp()

        # This is the return value for utils.find_resource()
        self.app_creds_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.APP_CRED_BASIC),
            loaded=True,
        )
        self.app_creds_mock.delete.return_value = None

        # Get the command object to test
        self.cmd = application_credential.DeleteApplicationCredential(
            self.app, None)

    def test_application_credential_delete(self):
        arglist = [
            identity_fakes.app_cred_id,
        ]
        verifylist = [
            ('application_credential', [identity_fakes.app_cred_id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.app_creds_mock.delete.assert_called_with(
            identity_fakes.app_cred_id,
        )
        self.assertIsNone(result)

    @mock.patch.object(utils, 'find_resource')
    def test_delete_multi_app_creds_with_exception(self, find_mock):
        find_mock.side_effect = [self.app_creds_mock.get.return_value,
                                 exceptions.CommandError]
        arglist = [
            identity_fakes.app_cred_id,
            'nonexistent_app_cred',
        ]
        verifylist = [
            ('application_credential', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        try:
            self.cmd.take_action(parsed_args)
            self.fail('CommandError should be raised.')
        except exceptions.CommandError as e:
            self.assertEqual('1 of 2 application credentials failed to'
                             ' delete.', str(e))

        find_mock.assert_any_call(self.app_creds_mock,
                                  identity_fakes.app_cred_id)
        find_mock.assert_any_call(self.app_creds_mock,
                                  'nonexistent_app_cred')

        self.assertEqual(2, find_mock.call_count)
        self.app_creds_mock.delete.assert_called_once_with(
            identity_fakes.app_cred_id)


class TestApplicationCredentialList(TestApplicationCredential):

    def setUp(self):
        super(TestApplicationCredentialList, self).setUp()

        self.app_creds_mock.list.return_value = [
            fakes.FakeResource(
                None,
                copy.deepcopy(identity_fakes.APP_CRED_BASIC),
                loaded=True,
            ),
        ]

        # Get the command object to test
        self.cmd = application_credential.ListApplicationCredential(self.app,
                                                                    None)

    def test_application_credential_list(self):
        arglist = []
        verifylist = []
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class Lister in cliff, abstract method take_action()
        # returns a tuple containing the column names and an iterable
        # containing the data to be listed.
        columns, data = self.cmd.take_action(parsed_args)

        self.app_creds_mock.list.assert_called_with(user=None)

        collist = ('ID', 'Name', 'Project ID', 'Description', 'Expires At')
        self.assertEqual(collist, columns)
        datalist = ((
            identity_fakes.app_cred_id,
            identity_fakes.app_cred_name,
            identity_fakes.project_id,
            None,
            None
        ), )
        self.assertEqual(datalist, tuple(data))


class TestApplicationCredentialShow(TestApplicationCredential):

    def setUp(self):
        super(TestApplicationCredentialShow, self).setUp()

        self.app_creds_mock.get.return_value = fakes.FakeResource(
            None,
            copy.deepcopy(identity_fakes.APP_CRED_BASIC),
            loaded=True,
        )

        # Get the command object to test
        self.cmd = application_credential.ShowApplicationCredential(self.app,
                                                                    None)

    def test_application_credential_show(self):
        arglist = [
            identity_fakes.app_cred_id,
        ]
        verifylist = [
            ('application_credential', identity_fakes.app_cred_id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # In base command class ShowOne in cliff, abstract method take_action()
        # returns a two-part tuple with a tuple of column names and a tuple of
        # data to be shown.
        columns, data = self.cmd.take_action(parsed_args)

        self.app_creds_mock.get.assert_called_with(identity_fakes.app_cred_id)

        collist = ('access_rules', 'description', 'expires_at', 'id', 'name',
                   'project_id', 'roles', 'secret', 'unrestricted')
        self.assertEqual(collist, columns)
        datalist = (
            None,
            None,
            None,
            identity_fakes.app_cred_id,
            identity_fakes.app_cred_name,
            identity_fakes.project_id,
            identity_fakes.role_name,
            identity_fakes.app_cred_secret,
            False,
        )
        self.assertEqual(datalist, data)



#   Copyright 2012-2013 OpenStack Foundation
#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from io import StringIO
import os

import fixtures
import testtools

from openstackclient.tests.unit import fakes


class ParserException(Exception):
    pass


class CompareBySet(list):
    """Class to compare value using set."""
    def __eq__(self, other):
        return set(self) == set(other)


class TestCase(testtools.TestCase):

    def setUp(self):
        testtools.TestCase.setUp(self)

        if (os.environ.get("OS_STDOUT_CAPTURE") == "True" or
                os.environ.get("OS_STDOUT_CAPTURE") == "1"):
            stdout = self.useFixture(fixtures.StringStream("stdout")).stream
            self.useFixture(fixtures.MonkeyPatch("sys.stdout", stdout))

        if (os.environ.get("OS_STDERR_CAPTURE") == "True" or
                os.environ.get("OS_STDERR_CAPTURE") == "1"):
            stderr = self.useFixture(fixtures.StringStream("stderr")).stream
            self.useFixture(fixtures.MonkeyPatch("sys.stderr", stderr))

    def assertNotCalled(self, m, msg=None):
        """Assert a function was not called"""

        if m.called:
            if not msg:
                msg = 'method %s should not have been called' % m
            self.fail(msg)


class TestCommand(TestCase):
    """Test cliff command classes"""

    def setUp(self):
        super(TestCommand, self).setUp()
        # Build up a fake app
        self.fake_stdout = fakes.FakeStdout()
        self.fake_log = fakes.FakeLog()
        self.app = fakes.FakeApp(self.fake_stdout, self.fake_log)
        self.app.client_manager = fakes.FakeClientManager()
        self.app.options = fakes.FakeOptions()

    def check_parser(self, cmd, args, verify_args):
        cmd_parser = cmd.get_parser('check_parser')
        stderr = StringIO()
        with fixtures.MonkeyPatch('sys.stderr', stderr):
            try:
                parsed_args = cmd_parser.parse_args(args)
            except SystemExit:
                raise ParserException("Argument parse failed: %s" %
                                      stderr.getvalue())
        for av in verify_args:
            attr, value = av
            if attr:
                self.assertIn(attr, parsed_args)
                self.assertEqual(value, getattr(parsed_args, attr))
        return parsed_args
